import numpy as np
from pyvstress import Point, SoilLayer, SoilProfile
import pyvstress.utility_functions as utf
from shapely import Polygon

# Description: Determine active lateral earth pressure profile and then compute the the 
# lateral earth pressure force at the base of the excavation at 35 ft depth
# we will use shapely to compute the area for now. I will add numerical integration to compute
# area when I can.
# Use the total force to compute the max. stress in FHWA and Peck apparent earth pressure diagram 


def calc_rankine_ka(phi):
    """Compute Rankines active earth pressure coefficient
    :param phi: float, Mohr-Coulomb phi parameter"""
    return np.tan(np.radians(45 - phi / 2)) ** 2


def calc_rankine_kp(phi):
    """Compute Rankines passive earth pressure coefficient
    :param c: float, Mohr-Coulomb c parameter"""
    return np.tan(np.radians(45 + phi / 2)) ** 2


def calc_active_earth_pressure(sigma, phi, c):
    """Active earth pressure using both phi and c
    :param sigma: float effective stress
    :param phi: Mohr-Coulomb phi parameter
    :param c: Mohr-Coulomb c parameter"""
    ka = calc_rankine_ka(phi)
    return sigma * ka - 2 * c * np.sqrt(ka)


def calc_passive_earth_pressure(sigma, phi, c):
    """Passive earth pressure using both phi and c
    :param sigma: float effective stress
    :param phi: Mohr-Coulomb phi parameter
    :param c: Mohr-Coulomb c parameter"""
    kp = calc_rankine_kp(phi)
    return sigma * kp + 2 * c * np.sqrt(kp)


def main():
   
    # Create layers with optional arguments for lateral earth pressure calculation
    # soil layer only till the excavation depth
    layer1 = SoilLayer(24, 125, 125, phi=30, c=0)
    layer2 = SoilLayer(11, 130, 130, phi=32, c=0)
    layers = [layer1, layer2]

    # excavation height, ft
    hexca = 24 + 11

    # create soil profile, with watertable at 5 feet from the ground surface and surcharge
    soilprofile = SoilProfile(layers, zw=7.0, q=0.0, gammaw=62.4)

    # Use the utility_functions profile_lateral_earth_pressures
    vstress, latstress = utf.profile_lateral_earth_pressures(soilprofile)
    # results = np.c_[vstress[:, 0], vstress[:, 3,], latstress[:,1:]]
    
    # # convert results to numpy array for easy manipulation
    # results = np.round(results, 1)
    

    np.set_printoptions(suppress=True)
    # print(np.around(results, decimals=2))
    
    # total active force
    coords = list(zip(latstress[:, 0], latstress[:, 1]))
    coords = coords + [(hexca, 0.0), (0.0, 0.0)]
    poly = Polygon(coords)
    total_active_force = poly.area

    print(f"Total active force: {total_active_force:.2f} lbf/ft")
    print(f"FHWA pressure p: {total_active_force/(2/3 * hexca) * 1.3:.2f} psf")
    print(f"Peck pressure p: {total_active_force/(0.5 * hexca) * 0.65:.2f} psf")

if __name__ == "__main__":
    main()
