import numpy as np
from pyvstress import SoilLayer, SoilProfile
import pyvstress.utility_functions as utf

# Description: keyword arguments for SoilLayer


def generate_regular_points(soil_profile, analysis_type="vertical", dz=0.5):
    """Generate a list of points within the soil profile layer.

    Generate points, when possible, at equal intervals of dz.
    Points may not be dz apart near layer boundaries. Either one or two points
    are generated at internal layer boundaries based on whether the type of
    analysis involves vertical stresses, 1 point per boundary, or lateral stresses
    2 points per boundary, one point for the layer above the layer boundary and one
    point for the layer below the boundary.

    :param SoilProfile soilprofile: SoilProfile object
    :param float dz: maximum separation distance between points
    :param str analysis_type: type of analysis either 'vertical' or 'lateral'
    :returns: a list of Point objects, almost equally spaced over the soil profile
    :rtype: [Point]
    """

    # generate boundary points
    ptb = utf.stress_profile_points(soil_profile, analysis_type)
    return ptb


def tz_func(side_disp: float) -> float:
    """T-z curve for clay.
    :param side_disp float: side displacement inch
    :returns:  ratio of the side shear for the applied displacement to the pile ultimate side shear capacity, unitless.
    :rtype: float
    """
    return 1.0 if side_disp >= 0.1 else 1 / 0.1 * side_disp


def qz_func(base_disp_ratio: float) -> float:
    """Q-z curve for clay.
    :param base_disp_ratio float: ratio of the base displacement to pile tip diameter
    :returns:  ratio of base displacement to pile tip diameter for the applied displacement, unitless.
    :rtype: float
    """
    return 1.0 if base_disp_ratio >= 0.01 else 1 / 0.01 * base_disp_ratio


def main():

    # soillayers
    layer1 = SoilLayer(
        5.0,
        110,
        120,
        phi=26,
        c=100,
        soil_type="clay",
        tz_curve=tz_func,
        qz_curve=qz_func,
    )
    layer2 = SoilLayer(
        7.0,
        115,
        125,
        phi=28,
        c=100,
        soil_type="clay",
        tz_curve=tz_func,
        qz_curve=qz_func,
    )
    layer3 = SoilLayer(
        4.0, 115, 122, phi=30, c=0, soil_type="clay", tz_curve=tz_func, qz_curve=qz_func
    )
    layers = [layer1, layer2, layer3]

    # soil profile
    soilprofile = SoilProfile(layers, zw=1.5, q=0, gammaw=62.4)
    pts = utf.generate_regular_points(soilprofile, "vertical", dz=0.5)

    # compute vertical total stress, porewater pressure, and effective stresses
    soilprofile.vertical_stresses(pts)
    soilprofile.assign_layer_index(pts)

    for pt in pts:
        print(soilprofile[pt.layer_index].tz_curve(0.02))
    # keylist = ["tz_curve", "qz_curve"]
    # params = soilprofile.get_params(pts, keylist)

    # for p in params:
    #     print(p[0](0.005), p[1](0.0025))

    # for layer in soilprofile:
    #     print(layer.tz_curve(0.05), layer.qz_curve(0.005))

    # for pt in pts:
    #     print(pt.z)

    # dz = 0.5
    # depth = soilprofile.zbots[-1]
    # drem = depth % 10

    # if drem > 0:
    #     depth = math.floor(depth)
    #     print("here-1")
    # else:
    #     depth = math.floor(depth)
    #     print("here-2")

    # npts = int(depth/dz) + 1
    # pti = [i for i in np.linspace(0, depth, npts)]
    # pts = list(set(pts).union(set(pti)))
    # pts.sort()
    # print(f"pts: {pts}")
    # print(f"pti: {pti}")
    # print(f"pt: {pts}")
    # # print(bti)


if __name__ == "__main__":
    main()
